"""Little helper application to improve django choices (for fields)"""

from __future__ import unicode_literals

from .choices import Choices, OrderedChoices

__author__ = 'Stephane "Twidi" Ange;'
__contact__ = "s.angel@twidi.com"
__homepage__ = "https://pypi.python.org/pypi/django-extended-choices"
__version__ = "1.1.1"
